/*
 * Copyright (c) 2024. Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.nachrichten.antragraum;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import jakarta.annotation.PostConstruct;

import org.opensaml.saml.security.impl.SAMLSignatureProfileValidator;
import org.opensaml.xmlsec.signature.support.SignatureTrustEngine;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.security.saml2.core.Saml2Error;
import org.springframework.security.saml2.core.Saml2ErrorCodes;
import org.springframework.stereotype.Service;

import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;
import net.shibboleth.utilities.java.support.resolver.CriteriaSet;

@Log4j2
@RequiredArgsConstructor
@Service
@ConditionalOnProperty(AntragraumProperties.PROPERTY_ANTRAGSRAUM_URL)
class Saml2Verifier {
	public static final String INVALID_SIGNATURE = "Invalid signature for object [%s]: ";
	public static final String SIGNATURE_MISSING = "Signature missing";

	private final Saml2Parser parser;
	private final BayernIdSamlConfiguration configuration;

	private SignatureTrustEngine trustEngine;
	private CriteriaSet verificationCriteria;

	@PostConstruct
	void init() {
		trustEngine = configuration.getTrustEngine();
		verificationCriteria = configuration.getVerificationCriteria();
	}

	List<Saml2Error> verify(String samlToken) {
		var response = parser.parse(samlToken);

		List<Saml2Error> errors = new ArrayList<>();
		var signature = response.getSignature();
		var profileValidator = new SAMLSignatureProfileValidator();
		if (Objects.nonNull(signature)) {
			try {
				profileValidator.validate(signature);
			} catch (Exception ex) {
				LOG.error("Error validating SAML Token: ", ex);
				errors.add(new Saml2Error(Saml2ErrorCodes.INVALID_SIGNATURE, INVALID_SIGNATURE.formatted(response.getID())));
			}

			try {
				if (!trustEngine.validate(signature, verificationCriteria)) {
					errors.add(new Saml2Error(Saml2ErrorCodes.INVALID_SIGNATURE, INVALID_SIGNATURE.formatted(response.getID())));
				}
			} catch (Exception ex) {
				LOG.error("Error validating SAML Token: ", ex);
				errors.add(new Saml2Error(Saml2ErrorCodes.INVALID_SIGNATURE, INVALID_SIGNATURE.formatted(response.getID())));
			}
		} else {
			errors.add(new Saml2Error(Saml2ErrorCodes.INVALID_SIGNATURE, SIGNATURE_MISSING));
		}

		return errors;
	}
}
