package de.ozgcloud.nachrichten.postfach.osi;

import java.util.List;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Named;
import org.mapstruct.ReportingPolicy;
import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.nachrichten.postfach.FileId;
import de.ozgcloud.nachrichten.postfach.PostfachAddress;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.StringBasedIdentifier;

@Mapper(unmappedTargetPolicy = ReportingPolicy.WARN)
public abstract class OsiPostfachMessageMapper {

	private static final String POSTFACH_ADDRESS_VERSION = "1.0";
	private static final int POSTFACH_ADDRESS_TYPE = 1;

	@Autowired
	private MessageAttachmentService messageAttachmentService;

	@Mapping(target = "attachment", ignore = true)
	@Mapping(target = "isHtml", constant = "true")
	@Mapping(target = "rechtsverbindlich", constant = "false")
	@Mapping(target = "eidasLevel", constant = "MEDIUM")
	@Mapping(target = "postfachId", expression = "java(toPostfachId(nachricht))")
	@Mapping(target = "mailBody", qualifiedByName = "replaceNewLine")
	public abstract Message toOsiMessage(PostfachNachricht nachricht);

	@Mapping(target = "createdBy", ignore = true)
	@Mapping(target = "id", ignore = true)
	@Mapping(target = "messageCode", ignore = true)
	@Mapping(target = "sentAt", ignore = true)
	@Mapping(target = "sentSuccessful", ignore = true)
	@Mapping(target = "createdAt", expression = "java(java.time.ZonedDateTime.now())")
	@Mapping(target = "direction", constant = "IN")
	@Mapping(target = "attachments", expression = "java(persistAttachements(message.getVorgangId() ,message.getAttachments()))")
	@Mapping(target = "postfachAddress", expression = "java(buildPostfachAddressByPostfachId(message.getPostfachId()))")
	@Mapping(target = "mailBody", qualifiedByName = "clearReceivedMessage")
	public abstract PostfachNachricht toPostfachNachricht(Message message);

	List<MessageAttachment> map(List<String> attachedFileIds) {
		return attachedFileIds.stream().map(FileId::from).map(fileId -> messageAttachmentService.getMessageAttachment(fileId)).toList();
	}

	List<String> persistAttachements(String vorgangId, List<MessageAttachment> attachments) {
		return attachments.stream().map(attachment -> messageAttachmentService.persistAttachment(vorgangId, attachment)).toList();
	}

	String toPostfachId(PostfachNachricht nachricht) {
		return Optional.ofNullable(nachricht.getPostfachAddress())
				.map(PostfachAddress::getIdentifier)
				.filter(StringBasedIdentifier.class::isInstance)
				.map(StringBasedIdentifier.class::cast)
				.map(StringBasedIdentifier::getPostfachId)
				.orElse(StringUtils.EMPTY);
	}

	PostfachAddress buildPostfachAddressByPostfachId(String postfachId) {
		return PostfachAddress.builder()
				.type(POSTFACH_ADDRESS_TYPE)
				.version(POSTFACH_ADDRESS_VERSION)
				.identifier(StringBasedIdentifier.builder().postfachId(postfachId).build())
				.serviceKontoType(OsiPostfachRemoteService.POSTFACH_TYPE)
				.build();

	}

	@Named("replaceNewLine")
	String replaceNewlines(String nachricht) {
		return Optional.ofNullable(nachricht).map(str -> str.replaceAll("(\r\n|\n)", "<br>")).orElse(null);
	}

	@Named("clearReceivedMessage")
	String clearReceivedMessage(String nachricht) {
		return Optional.ofNullable(nachricht).map(str -> str.replace("&amp;", "&")).orElse(null);
	}
}