/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.context.ApplicationEventPublisher;

import de.ozgcloud.command.Command;
import de.ozgcloud.command.CommandCreatedEvent;
import de.ozgcloud.command.PostfachNachrichtReceivedEvent;
import de.ozgcloud.command.VorgangAttachedItemCreatedEvent;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht.Direction;
import de.ozgcloud.nachrichten.postfach.osi.MessageTestFactory;

class PostfachEventListenerTest {

	@InjectMocks
	private PostfachEventListener listener;

	@Mock
	private PostfachService service;
	@Mock
	private Command command;
	@Mock
	private ApplicationEventPublisher publisher;

	@Captor
	private ArgumentCaptor<PostfachNachricht> nachrichtCaptor;

	@DisplayName("Send postfach nachricht")
	@Nested
	class TestSendPostfachNachricht {

		private static final String COMMAND_ID = UUID.randomUUID().toString();
		private static final String USER_ID = PostfachNachrichtTestFactory.CREATED_BY;

		@BeforeEach
		void mockCommand() {
			when(command.getId()).thenReturn(COMMAND_ID);
			when(command.getCreatedBy()).thenReturn(USER_ID);
			when(command.getVorgangId()).thenReturn(MessageTestFactory.VORGANG_ID);
			when(command.getRelationId()).thenReturn(MessageTestFactory.VORGANG_ID);
			when(command.getBodyObject()).thenReturn(buildCommandBody());
		}

		@Test
		void shouldCallService() {
			listener.sendPostfachNachricht(new CommandCreatedEvent(command));

			verify(service).sendMail(eq(COMMAND_ID), eq(USER_ID), nachrichtCaptor.capture());
			assertThat(nachrichtCaptor.getValue())
					.usingRecursiveComparison()
					.ignoringFields("id", "createdAt", "messageCode", "messageId", "sentAt", "sentSuccessful", "referencedNachricht")
					.isEqualTo(PostfachNachrichtTestFactory.createBuilder().direction(Direction.OUT).build());
		}

		@Test
		void shouldNotHaveIdForNewNachricht() {
			listener.sendPostfachNachricht(new CommandCreatedEvent(command));

			verify(service).sendMail(eq(COMMAND_ID), eq(USER_ID), nachrichtCaptor.capture());
			assertThat(nachrichtCaptor.getValue().getId()).isNull();
		}

		@Test
		void shouldHaveIdForSavedNachricht() {
			when(command.getRelationId()).thenReturn(PostfachNachrichtTestFactory.ID);

			listener.sendPostfachNachricht(new CommandCreatedEvent(command));

			verify(service).sendMail(eq(COMMAND_ID), eq(USER_ID), nachrichtCaptor.capture());
			assertThat(nachrichtCaptor.getValue().getId()).isEqualTo(PostfachNachrichtTestFactory.ID);
		}

		@DisplayName("with postfachId")
		@Nested
		class TestWithPostfachId {

			@BeforeEach
			void mockCommandBodyObject() {
				var body = buildCommandBody();
				body.remove(PostfachNachricht.POSTFACH_ADDRESS_FIELD);
				when(command.getBodyObject()).thenReturn(body);
			}

			@Test
			void shouldHandleEmptyPostfachAddress() {
				var postfachMailToSend = sendPostfachNachricht();

				assertThat(postfachMailToSend.getPostfachAddress()).isNull();
			}
		}

		private PostfachNachricht sendPostfachNachricht() {
			listener.sendPostfachNachricht(new CommandCreatedEvent(command));

			verify(service).sendMail(eq(COMMAND_ID), eq(USER_ID), nachrichtCaptor.capture());

			return nachrichtCaptor.getValue();
		}

		private Map<String, Object> buildCommandBody() {
			var map = new HashMap<String, Object>();
			map.putAll(Map.of(
					PostfachNachricht.FIELD_VORGANG_ID, "fake",
					PostfachNachricht.POSTFACH_ADDRESS_FIELD, //
					Map.of(PostfachAddress.VERSION_FIELD, PostfachAddressTestFactory.VERSION,
							PostfachAddress.TYPE_FIELD, PostfachAddressTestFactory.TYPE,
							PostfachAddress.SERVICEKONTO_TYPE_FIELD, PostfachTestFactory.POSTFACH_TYPE,
							PostfachAddress.IDENTIFIER_FIELD, //
							Map.of(PostfachAddress.FIELD_POSTFACH_ID, PostfachAddressTestFactory.STRING_BASED_IDENTIFIER_POSTFACH_ID_VALUE)),
					PostfachNachricht.FIELD_SUBJECT, MessageTestFactory.SUBJECT,
					PostfachNachricht.FIELD_MAIL_BODY, PostfachNachrichtTestFactory.MAIL_BODY,
					PostfachNachricht.FIELD_REPLY_OPTION, PostfachNachrichtTestFactory.REPLY_OPTION.name(),
					PostfachNachricht.FIELD_ATTACHMENTS, PostfachNachrichtTestFactory.ATTACHMENTS));
			return map;
		}
	}

	@DisplayName("Get string list value")
	@Nested
	class TestGetStringListValue {

		@Test
		void shouldReturnMultipleValuesFieldAsStringList() {
			var commandBodyMap = new HashMap<String, Object>();
			commandBodyMap.put(PostfachNachricht.FIELD_ATTACHMENTS, List.of("1", "2"));

			var list = listener.getStringListValue(commandBodyMap, PostfachNachricht.FIELD_ATTACHMENTS);

			assertThat(list).contains("1").contains("2");
		}

		@Test
		void shouldReturnSingleValueFieldAsStringList() {
			var commandBodyMap = new HashMap<String, Object>();
			commandBodyMap.put(PostfachNachricht.FIELD_ATTACHMENTS, List.of("1", "2"));

			var list = listener.getStringListValue(commandBodyMap, PostfachNachricht.FIELD_ATTACHMENTS);

			assertThat(list).contains("1").contains("2");
		}

		@Test
		void shouldReturnEmptyFieldValueAsEmptyCollection() {
			var commandBodyMap = new HashMap<String, Object>();
			commandBodyMap.put(PostfachNachricht.FIELD_ATTACHMENTS, Collections.emptyList());

			var list = listener.getStringListValue(commandBodyMap, PostfachNachricht.FIELD_ATTACHMENTS);

			assertThat(list).isEmpty();
		}

		@Test
		void shouldReturnEmptyFieldAsEmptyCollection() {
			var commandBodyMap = new HashMap<String, Object>();

			var list = listener.getStringListValue(commandBodyMap, PostfachNachricht.FIELD_ATTACHMENTS);

			assertThat(list).isEmpty();
		}
	}

	@DisplayName("On create incoming nachricht item")
	@Nested
	class TestOnCreateIncomingPostfachNachrichtItem {

		private static final String CREATED_RESOURCE = "created-resource";
		@Captor
		public ArgumentCaptor<PostfachNachrichtReceivedEvent> eventCaptor;
		@Mock
		private VorgangAttachedItemCreatedEvent vorgangAttachedItemCreatedEvent;

		@BeforeEach
		void init() {
			when(vorgangAttachedItemCreatedEvent.getCommand()).thenReturn(command);
			when(vorgangAttachedItemCreatedEvent.getCreatedResource()).thenReturn(CREATED_RESOURCE);
		}

		@Test
		void shouldPublishPostfachNachrichtReceivedEvent() {
			listener.onIncomingPostfachNachrichtItemCreated(vorgangAttachedItemCreatedEvent);

			verify(publisher).publishEvent(eventCaptor.capture());
			assertThat(eventCaptor.getValue()).satisfies(event -> {
				assertThat(event.getSource()).isSameAs(command);
				assertThat(event.getNachrichtId()).isEqualTo(CREATED_RESOURCE);
			});
		}
	}
}